/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* $Id: $ */

package org.apache.fop.render.afp.modca;

import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;

import org.apache.fop.render.afp.bcoca.BarcodeSymbologies;
import org.apache.fop.render.afp.bcoca.LinearSymbologies;
import org.apache.fop.render.afp.tools.BinaryUtils;

/**
 * The Barcode Data (BDA) structured field contains parameters to position a
 * single bar code symbol within a bar code presentation space, parameters to
 * specify special functions for 2D bar codes, flags to specify attributes
 * specific to the symbol, and the data to be encoded. The data is encoded
 * according to the parameters specified in the Bar Code Data Descriptor (BDD)
 * structured field
 */
public class BarcodeData extends AbstractAFPObject {

	/**
	 * Dimensions of the barcode presentation space
	 */
	private BarcodeSymbologies _bcocaObject;

	/**
	 * The barcode data
	 */
	byte[] _barcodeData = null;

	/**
	 * Default constructor for the BarcodeData.
	 * 
	 * @param data
	 *            the barcode data
	 * @param bcoca
	 *            the barcode attributes
	 */
	public BarcodeData(String data, BarcodeSymbologies bcoca) {

		_bcocaObject = bcoca;

		try {
			_barcodeData = data.getBytes(_bcocaObject.getDataEncoding());
		} catch (UnsupportedEncodingException usee) {
			_barcodeData = data.getBytes();
			log.warn("Constructor:: UnsupportedEncodingException translating the barcode data: " + data);
		}

	}

	/**
	 * Accessor method to write the AFP datastream for the barcode data.
	 * 
	 * @param os
	 *            The stream to write to
	 * @throws java.io.IOException
	 */
	public void writeDataStream(OutputStream os) throws IOException {

		byte[] specialFunctions = _bcocaObject.getSpecialFunctions();
		int length = 14 + specialFunctions.length + _barcodeData.length;
		byte[] data = new byte[length];

		data[0] = 0x5A;
		// Set the total record length
		byte[] rl1 = BinaryUtils.convert(length - 1, 2);
		data[1] = rl1[0];
		data[2] = rl1[1];

		// Structured field the BarcodeData
		data[3] = (byte) 0xD3;
		data[4] = (byte) 0xEE;
		data[5] = (byte) 0xEB;

		data[6] = 0x00; // Reserved
		data[7] = 0x00; // Reserved
		data[8] = 0x00; // Reserved

		byte flags = (byte) 0x00;
		
		if (_bcocaObject instanceof LinearSymbologies) {
			LinearSymbologies linearSymbologies = (LinearSymbologies) _bcocaObject;
			flags = BinaryUtils.setBit(flags, 0, !linearSymbologies.isHRIPresented());
			if (linearSymbologies.isHRIPositionAbove()) {
				// "10" Above
				flags = BinaryUtils.setBit(flags, 1, true);
				flags = BinaryUtils.setBit(flags, 2, false);
			} else if (linearSymbologies.isHRIPositionBelow()) {
				// "01" Below
				flags = BinaryUtils.setBit(flags, 1, false);
				flags = BinaryUtils.setBit(flags, 2, true);
			} else {
				// Use the default "00"
			}
			flags = BinaryUtils.setBit(flags, 3, linearSymbologies.isAsteriskPresented());
		} else {
			// Asterisk not presented
			flags = BinaryUtils.setBit(flags, 3, false);
		}

		flags = BinaryUtils.setBit(flags, 5, _bcocaObject.isSymbolSupressed());
		data[9] = flags;

		int xOffset = _bcocaObject.getXOffset();
		int yOffset = _bcocaObject.getYOffset();

		if (xOffset < 1) {
			data[10] = 0x00;
			data[11] = 0x01;
		} else {
			byte[] xOset = BinaryUtils.convert(xOffset, 2);
			data[10] = xOset[0];
			data[11] = xOset[1];
		}

		if (yOffset < 1) {
			data[12] = 0x00;
			data[13] = 0x01;
		} else {
			byte[] yOset = BinaryUtils.convert(yOffset, 2);
			data[12] = yOset[0];
			data[13] = yOset[1];
		}

		System
				.arraycopy(specialFunctions, 0, data, 14,
						specialFunctions.length);
		System.arraycopy(_barcodeData, 0, data, 14 + specialFunctions.length,
				_barcodeData.length);

		os.write(data);

	}

}