/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* $Id: $ */

package org.apache.fop.render.afp.modca;

import java.io.IOException;
import java.io.OutputStream;

import org.apache.fop.render.afp.bcoca.BarcodeSymbologies;
import org.apache.fop.render.afp.bcoca.StackedSymbologies;
import org.apache.fop.render.afp.tools.BinaryUtils;

/**
 * The Barcode Data Descriptor (BDA) specifies the size of the bar code presentation space,
 * the type of bar code to be generated, and the parameters used to generate the bar code symbols.
 */
public class BarcodeDataDescriptor implements DataDescriptor {
	
	/**
	 * The barcode attributes
	 */
	private BarcodeSymbologies _bcocaObject;
	
    /**
     * Constructor for a BarcodeDataDescriptor
     * @param bcoca the barcode attributes
     */
    public BarcodeDataDescriptor(BarcodeSymbologies bcoca) {

    	_bcocaObject = bcoca;

    }

    /**
     * Accessor method to write the AFP datastream for the Image Data Descriptor
     * @param os The stream to write to
     * @throws java.io.IOException
     */
    public void writeDataStream(OutputStream os)
        throws IOException {
    	
    	byte[] unitsperbase = new byte[2];
    	if (_bcocaObject.getMpts2unitsFactor() == 50) {
    		unitsperbase[0] = 0x38;
    		unitsperbase[1] = 0x40;
    	} else if (_bcocaObject.getMpts2unitsFactor() == 300) {
    		unitsperbase[0] = 0x09;
    		unitsperbase[1] = 0x60;
    	}

    	byte[] bw = BinaryUtils.convert(_bcocaObject.getWidth(), 2);
    	byte[] bl = BinaryUtils.convert(_bcocaObject.getHeight(), 2);
    	
    	byte[] elementHeight = new byte[2];
    	byte[] heightMultiplier = new byte[1];
    	if (_bcocaObject instanceof StackedSymbologies) {
    		StackedSymbologies stackedSymbology = (StackedSymbologies) _bcocaObject;
    		int ehi = stackedSymbology.getElementHeight();
    		if (ehi < 1) {
        		// Use default
        		elementHeight[0] = (byte)0xFF;
        		elementHeight[1] = (byte)0xFF;
        		heightMultiplier[0] = 0x01;
        	} else {
        		elementHeight = BinaryUtils.convert(ehi, 2);
        		heightMultiplier = BinaryUtils.convert(stackedSymbology.getHeightMultiplier(), 2);
        	}
    	} else {
    		elementHeight[0] = (byte)0xFF;
    		elementHeight[1] = (byte)0xFF;
    		heightMultiplier[0] = 0x01;
    	}
    	
        byte[] data = new byte[] {
            0x5A,
            0x00,
            0x20,
            (byte) 0xD3,
            (byte) 0xA6,
            (byte) 0xEB,
            0x00, // Flags
            0x00, // Reserved
            0x00, // Reserved
            
            0x00, // Base units (ten inches)
            0x00, // Reserved
            
            unitsperbase[0], // Units per base in the X direction
            unitsperbase[1], // 
            unitsperbase[0], // Units per base in the Y direction
            unitsperbase[1], // 

            bw[0],
            bw[1],
            bl[0],
            bl[1],
            	
            0x00, // Reserved
            0x00, // Reserved
          
            _bcocaObject.getBarCodeType(),
            _bcocaObject.getBarCodeModifier(),
            
            (byte)0xFF, 			// Default HRI Local Font 
            (byte)0x00, 			// Default Color
            (byte)0x00, 			// Default Color
            
            BinaryUtils.convert(_bcocaObject.getModuleWidth(), 1)[0],       // The module width
            
            elementHeight[0], 		// Element height 
            elementHeight[1], 		// Element height  
            heightMultiplier[0],    // Default height multiplier
            
            (byte)0xFF, 			// Default wide to narrow ratio 
            (byte)0xFF, 			// Default wide to narrow ratio 
            
        };
        
        byte[] l = BinaryUtils.convert(data.length - 1, 2);
        data[1] = l[0];
        data[2] = l[1];

        os.write(data);

    }

}
