/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* $Id: $ */

package org.apache.fop.render.afp.modca;

import java.io.IOException;
import java.io.OutputStream;

import org.apache.fop.render.afp.bcoca.BarcodeSymbologies;

/**
 * Barcodes consists of patterns of bars and spaces that represent alphanumeric information. 
 * Characteristics of the patterns are defined by specific bar code symbologies. A bar code object 
 * carries the alphanumeric information that is to be presented as a bar code and the controls to 
 * present this information using a specific bar code symbology. The bar code data object is defined 
 * by the Bar Code Object Content Architecture.
 */
public class BarcodeObject extends AbstractNamedAFPObject {
	
	/**
     * Default name for the object environment group
     */
    private static final String DEFAULT_NAME = "BCO00001";
    
    private BarcodeSymbologies _bcocaObject;

    /**
     * The barcode object environment group
     */
    private BarcodeObjectEnvironmentGroup _barCodeObjectEnvironmentGroup = null;

    /**
     * The barcode object environment group
     */
    private BarcodeData _barCodeData = null;

 
    /**
     * Constructor for the barcode object
     * @param bean the barcode attributes
     * @param dimension the barcode dimension
     */
    public BarcodeObject(BarcodeSymbologies bcoca) {

        super(DEFAULT_NAME);
        _bcocaObject = bcoca;
        
    }
    
    /**
     * Sets the barcode display area position and size.
     *
     * @param x
     *            the x position of the barcode
     * @param y
     *            the y position of the barcode
     * @param r
     *            the rotation of the barcode
     */
    public void setBarcodeViewport(int x, int y, int r) {
        if (_barCodeObjectEnvironmentGroup == null) {
            _barCodeObjectEnvironmentGroup = new BarcodeObjectEnvironmentGroup(_bcocaObject);
        }
        _barCodeObjectEnvironmentGroup.setObjectArea(x, y, r);
    }

    /**
     * Set the data of the barcode.
     * @param data the barcode data
     */
    public void setBarCodeData(BarcodeData barcodeData) {
    	
    	_barCodeData = barcodeData;
    	
    }


    /**
     * Accessor method to write the AFP datastream for the Image Object
     * @param os The stream to write to
     * @throws java.io.IOException
     */
    public void writeDataStream(OutputStream os)
        throws IOException {

        writeStart(os);

        if (_barCodeObjectEnvironmentGroup != null) {
            _barCodeObjectEnvironmentGroup.writeDataStream(os);
        }

        if (_barCodeData != null) {
        	_barCodeData.writeDataStream(os);
        }

        writeEnd(os);

    }

    /**
     * Helper method to write the start of the Image Object.
     * @param os The stream to write to
     */
    private void writeStart(OutputStream os)
        throws IOException {

        byte[] data = new byte[17];

        data[0] = 0x5A; // Structured field identifier
        data[1] = 0x00; // Length byte 1
        data[2] = 0x10; // Length byte 2
        data[3] = (byte) 0xD3; // Structured field id byte 1
        data[4] = (byte) 0xA8; // Structured field id byte 2
        data[5] = (byte) 0xEB; // Structured field id byte 3
        data[6] = 0x00; // Flags
        data[7] = 0x00; // Reserved
        data[8] = 0x00; // Reserved

        for (int i = 0; i < _nameBytes.length; i++) {

            data[9 + i] = _nameBytes[i];

        }

        os.write(data);

    }

    /**
     * Helper method to write the end of the Image Object.
     * @param os The stream to write to
     */
    private void writeEnd(OutputStream os)
        throws IOException {

        byte[] data = new byte[17];

        data[0] = 0x5A; // Structured field identifier
        data[1] = 0x00; // Length byte 1
        data[2] = 0x10; // Length byte 2
        data[3] = (byte) 0xD3; // Structured field id byte 1
        data[4] = (byte) 0xA9; // Structured field id byte 2
        data[5] = (byte) 0xEB; // Structured field id byte 3
        data[6] = 0x00; // Flags
        data[7] = 0x00; // Reserved
        data[8] = 0x00; // Reserved

        for (int i = 0; i < _nameBytes.length; i++) {

            data[9 + i] = _nameBytes[i];

        }

        os.write(data);

    }

}
